/*
  Liquid War 6 is a unique multiplayer wargame.
  Copyright (C)  2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015  Christian Mauduit <ufoot@ufoot.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.


  Liquid War 6 homepage : http://www.gnu.org/software/liquidwar6/
  Contact author        : ufoot@ufoot.org
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <string.h>

#include "map.h"

const char *LW6MAP_RULES_LIST[] = {
  LW6DEF_TOTAL_TIME,
  LW6DEF_RESPAWN_TEAM,
  LW6DEF_RESPAWN_POSITION_MODE,
  LW6DEF_RESPAWN_DELAY,
  LW6DEF_MOVES_PER_ROUND,
  LW6DEF_SPREADS_PER_ROUND,
  LW6DEF_ROUNDS_PER_SEC,
  LW6DEF_FIGHTER_ATTACK,
  LW6DEF_FIGHTER_DEFENSE,
  LW6DEF_FIGHTER_NEW_HEALTH,
  LW6DEF_FIGHTER_REGENERATE,
  LW6DEF_SIDE_ATTACK_FACTOR,
  LW6DEF_SIDE_DEFENSE_FACTOR,
  LW6DEF_NB_MOVE_TRIES,
  LW6DEF_NB_ATTACK_TRIES,
  LW6DEF_NB_DEFENSE_TRIES,
  LW6DEF_VERTICAL_MOVE,
  LW6DEF_SPREAD_MODE,
  LW6DEF_SINGLE_ARMY_SIZE,
  LW6DEF_TOTAL_ARMIES_SIZE,
  LW6DEF_MAX_NB_TEAMS,
  LW6DEF_MAX_NB_CURSORS,
  LW6DEF_MAX_NB_NODES,
  LW6DEF_EXP,
  LW6DEF_HIGHEST_TEAM_COLOR_ALLOWED,
  LW6DEF_HIGHEST_WEAPON_ALLOWED,
  LW6DEF_X_POLARITY,
  LW6DEF_Y_POLARITY,
  LW6DEF_Z_POLARITY,
  LW6DEF_MAX_ZONE_SIZE,
  LW6DEF_ROUND_DELTA,
  LW6DEF_MAX_ROUND_DELTA,
  LW6DEF_MAX_CURSOR_POT,
  LW6DEF_CURSOR_POT_INIT,
  LW6DEF_MAX_CURSOR_POT_OFFSET,
  LW6DEF_START_RED_X,
  LW6DEF_START_GREEN_X,
  LW6DEF_START_BLUE_X,
  LW6DEF_START_YELLOW_X,
  LW6DEF_START_CYAN_X,
  LW6DEF_START_MAGENTA_X,
  LW6DEF_START_ORANGE_X,
  LW6DEF_START_LIGHTBLUE_X,
  LW6DEF_START_PURPLE_X,
  LW6DEF_START_PINK_X,
  LW6DEF_START_RED_Y,
  LW6DEF_START_GREEN_Y,
  LW6DEF_START_BLUE_Y,
  LW6DEF_START_YELLOW_Y,
  LW6DEF_START_CYAN_Y,
  LW6DEF_START_MAGENTA_Y,
  LW6DEF_START_ORANGE_Y,
  LW6DEF_START_LIGHTBLUE_Y,
  LW6DEF_START_PURPLE_Y,
  LW6DEF_START_PINK_Y,
  LW6DEF_START_POSITION_MODE,
  LW6DEF_COLOR_CONFLICT_MODE,
  LW6DEF_SPREAD_THREAD,
  LW6DEF_GLUE_POWER,
  LW6DEF_BOOST_POWER,
  LW6DEF_DANGER_POWER,
  LW6DEF_MEDICINE_POWER,
  LW6DEF_FRAGS_MODE,
  LW6DEF_FRAGS_TO_DISTRIBUTE,
  LW6DEF_FRAGS_FADE_OUT,
  LW6DEF_USE_TEAM_PROFILES,
  LW6DEF_TEAM_PROFILE_RED_AGGRESSIVE,
  LW6DEF_TEAM_PROFILE_GREEN_AGGRESSIVE,
  LW6DEF_TEAM_PROFILE_BLUE_AGGRESSIVE,
  LW6DEF_TEAM_PROFILE_YELLOW_AGGRESSIVE,
  LW6DEF_TEAM_PROFILE_CYAN_AGGRESSIVE,
  LW6DEF_TEAM_PROFILE_MAGENTA_AGGRESSIVE,
  LW6DEF_TEAM_PROFILE_ORANGE_AGGRESSIVE,
  LW6DEF_TEAM_PROFILE_LIGHTBLUE_AGGRESSIVE,
  LW6DEF_TEAM_PROFILE_PURPLE_AGGRESSIVE,
  LW6DEF_TEAM_PROFILE_PINK_AGGRESSIVE,
  LW6DEF_TEAM_PROFILE_RED_VULNERABLE,
  LW6DEF_TEAM_PROFILE_GREEN_VULNERABLE,
  LW6DEF_TEAM_PROFILE_BLUE_VULNERABLE,
  LW6DEF_TEAM_PROFILE_YELLOW_VULNERABLE,
  LW6DEF_TEAM_PROFILE_CYAN_VULNERABLE,
  LW6DEF_TEAM_PROFILE_MAGENTA_VULNERABLE,
  LW6DEF_TEAM_PROFILE_ORANGE_VULNERABLE,
  LW6DEF_TEAM_PROFILE_LIGHTBLUE_VULNERABLE,
  LW6DEF_TEAM_PROFILE_PURPLE_VULNERABLE,
  LW6DEF_TEAM_PROFILE_PINK_VULNERABLE,
  LW6DEF_TEAM_PROFILE_RED_MOBILE,
  LW6DEF_TEAM_PROFILE_GREEN_MOBILE,
  LW6DEF_TEAM_PROFILE_BLUE_MOBILE,
  LW6DEF_TEAM_PROFILE_YELLOW_MOBILE,
  LW6DEF_TEAM_PROFILE_CYAN_MOBILE,
  LW6DEF_TEAM_PROFILE_MAGENTA_MOBILE,
  LW6DEF_TEAM_PROFILE_ORANGE_MOBILE,
  LW6DEF_TEAM_PROFILE_LIGHTBLUE_MOBILE,
  LW6DEF_TEAM_PROFILE_PURPLE_MOBILE,
  LW6DEF_TEAM_PROFILE_PINK_MOBILE,
  LW6DEF_TEAM_PROFILE_RED_FAST,
  LW6DEF_TEAM_PROFILE_GREEN_FAST,
  LW6DEF_TEAM_PROFILE_BLUE_FAST,
  LW6DEF_TEAM_PROFILE_YELLOW_FAST,
  LW6DEF_TEAM_PROFILE_CYAN_FAST,
  LW6DEF_TEAM_PROFILE_MAGENTA_FAST,
  LW6DEF_TEAM_PROFILE_ORANGE_FAST,
  LW6DEF_TEAM_PROFILE_LIGHTBLUE_FAST,
  LW6DEF_TEAM_PROFILE_PURPLE_FAST,
  LW6DEF_TEAM_PROFILE_PINK_FAST,
  LW6DEF_TEAM_PROFILE_RED_HANDICAP,
  LW6DEF_TEAM_PROFILE_GREEN_HANDICAP,
  LW6DEF_TEAM_PROFILE_BLUE_HANDICAP,
  LW6DEF_TEAM_PROFILE_YELLOW_HANDICAP,
  LW6DEF_TEAM_PROFILE_CYAN_HANDICAP,
  LW6DEF_TEAM_PROFILE_MAGENTA_HANDICAP,
  LW6DEF_TEAM_PROFILE_ORANGE_HANDICAP,
  LW6DEF_TEAM_PROFILE_LIGHTBLUE_HANDICAP,
  LW6DEF_TEAM_PROFILE_PURPLE_HANDICAP,
  LW6DEF_TEAM_PROFILE_PINK_HANDICAP,
  LW6DEF_TEAM_PROFILE_RED_WEAPON_ID,
  LW6DEF_TEAM_PROFILE_GREEN_WEAPON_ID,
  LW6DEF_TEAM_PROFILE_BLUE_WEAPON_ID,
  LW6DEF_TEAM_PROFILE_YELLOW_WEAPON_ID,
  LW6DEF_TEAM_PROFILE_CYAN_WEAPON_ID,
  LW6DEF_TEAM_PROFILE_MAGENTA_WEAPON_ID,
  LW6DEF_TEAM_PROFILE_ORANGE_WEAPON_ID,
  LW6DEF_TEAM_PROFILE_LIGHTBLUE_WEAPON_ID,
  LW6DEF_TEAM_PROFILE_PURPLE_WEAPON_ID,
  LW6DEF_TEAM_PROFILE_PINK_WEAPON_ID,
  LW6DEF_TEAM_PROFILE_RED_WEAPON_ALTERNATE_ID,
  LW6DEF_TEAM_PROFILE_GREEN_WEAPON_ALTERNATE_ID,
  LW6DEF_TEAM_PROFILE_BLUE_WEAPON_ALTERNATE_ID,
  LW6DEF_TEAM_PROFILE_YELLOW_WEAPON_ALTERNATE_ID,
  LW6DEF_TEAM_PROFILE_CYAN_WEAPON_ALTERNATE_ID,
  LW6DEF_TEAM_PROFILE_MAGENTA_WEAPON_ALTERNATE_ID,
  LW6DEF_TEAM_PROFILE_ORANGE_WEAPON_ALTERNATE_ID,
  LW6DEF_TEAM_PROFILE_LIGHTBLUE_WEAPON_ALTERNATE_ID,
  LW6DEF_TEAM_PROFILE_PURPLE_WEAPON_ALTERNATE_ID,
  LW6DEF_TEAM_PROFILE_PINK_WEAPON_ALTERNATE_ID,
  LW6DEF_TEAM_PROFILE_RED_WEAPON_MODE,
  LW6DEF_TEAM_PROFILE_GREEN_WEAPON_MODE,
  LW6DEF_TEAM_PROFILE_BLUE_WEAPON_MODE,
  LW6DEF_TEAM_PROFILE_YELLOW_WEAPON_MODE,
  LW6DEF_TEAM_PROFILE_CYAN_WEAPON_MODE,
  LW6DEF_TEAM_PROFILE_MAGENTA_WEAPON_MODE,
  LW6DEF_TEAM_PROFILE_ORANGE_WEAPON_MODE,
  LW6DEF_TEAM_PROFILE_LIGHTBLUE_WEAPON_MODE,
  LW6DEF_TEAM_PROFILE_PURPLE_WEAPON_MODE,
  LW6DEF_TEAM_PROFILE_PINK_WEAPON_MODE,
  LW6DEF_WEAPON_DURATION,
  LW6DEF_WEAPON_CHARGE_DELAY,
  LW6DEF_WEAPON_CHARGE_MAX,
  LW6DEF_WEAPON_TUNE_BERZERK_POWER,
  LW6DEF_WEAPON_TUNE_TURBO_POWER,
  NULL				// very important that this list is NULL-terminated
};

static lw6map_rules_t default_rules = {
  LW6MAP_RULES_DEFAULT_TOTAL_TIME,
  LW6MAP_RULES_DEFAULT_RESPAWN_TEAM,
  LW6MAP_RULES_DEFAULT_RESPAWN_POSITION_MODE,
  LW6MAP_RULES_DEFAULT_RESPAWN_DELAY,
  LW6MAP_RULES_DEFAULT_MOVES_PER_ROUND,
  LW6MAP_RULES_DEFAULT_SPREADS_PER_ROUND,
  LW6MAP_RULES_DEFAULT_ROUNDS_PER_SEC,
  LW6MAP_RULES_DEFAULT_FIGHTER_ATTACK,
  LW6MAP_RULES_DEFAULT_FIGHTER_DEFENSE,
  LW6MAP_RULES_DEFAULT_FIGHTER_NEW_HEALTH,
  LW6MAP_RULES_DEFAULT_FIGHTER_REGENERATE,
  LW6MAP_RULES_DEFAULT_SIDE_ATTACK_FACTOR,
  LW6MAP_RULES_DEFAULT_SIDE_DEFENSE_FACTOR,
  LW6MAP_RULES_DEFAULT_NB_MOVE_TRIES,
  LW6MAP_RULES_DEFAULT_NB_ATTACK_TRIES,
  LW6MAP_RULES_DEFAULT_NB_DEFENSE_TRIES,
  LW6MAP_RULES_DEFAULT_VERTICAL_MOVE,
  LW6MAP_RULES_DEFAULT_SPREAD_MODE,
  LW6MAP_RULES_DEFAULT_SINGLE_ARMY_SIZE,
  LW6MAP_RULES_DEFAULT_TOTAL_ARMIES_SIZE,
  LW6MAP_RULES_DEFAULT_MAX_NB_TEAMS,
  LW6MAP_RULES_DEFAULT_MAX_NB_CURSORS,
  LW6MAP_RULES_DEFAULT_MAX_NB_NODES,
  LW6MAP_RULES_DEFAULT_EXP,
  LW6MAP_RULES_DEFAULT_HIGHEST_TEAM_COLOR_ALLOWED,
  LW6MAP_RULES_DEFAULT_HIGHEST_WEAPON_ALLOWED,
  LW6MAP_RULES_DEFAULT_X_POLARITY,
  LW6MAP_RULES_DEFAULT_Y_POLARITY,
  LW6MAP_RULES_DEFAULT_Z_POLARITY,
  LW6MAP_RULES_DEFAULT_MAX_ZONE_SIZE,
  LW6MAP_RULES_DEFAULT_ROUND_DELTA,
  LW6MAP_RULES_DEFAULT_MAX_ROUND_DELTA,
  LW6MAP_RULES_DEFAULT_MAX_CURSOR_POT,
  LW6MAP_RULES_DEFAULT_CURSOR_POT_INIT,
  LW6MAP_RULES_DEFAULT_MAX_CURSOR_POT_OFFSET,
  // START_COLOR_X
  {
   LW6MAP_RULES_DEFAULT_START_RED_X,
   LW6MAP_RULES_DEFAULT_START_GREEN_X,
   LW6MAP_RULES_DEFAULT_START_BLUE_X,
   LW6MAP_RULES_DEFAULT_START_YELLOW_X,
   LW6MAP_RULES_DEFAULT_START_CYAN_X,
   LW6MAP_RULES_DEFAULT_START_MAGENTA_X,
   LW6MAP_RULES_DEFAULT_START_ORANGE_X,
   LW6MAP_RULES_DEFAULT_START_LIGHTBLUE_X,
   LW6MAP_RULES_DEFAULT_START_PURPLE_X,
   LW6MAP_RULES_DEFAULT_START_PINK_X},
  // START_COLOR_Y
  {
   LW6MAP_RULES_DEFAULT_START_RED_Y,
   LW6MAP_RULES_DEFAULT_START_GREEN_Y,
   LW6MAP_RULES_DEFAULT_START_BLUE_Y,
   LW6MAP_RULES_DEFAULT_START_YELLOW_Y,
   LW6MAP_RULES_DEFAULT_START_CYAN_Y,
   LW6MAP_RULES_DEFAULT_START_MAGENTA_Y,
   LW6MAP_RULES_DEFAULT_START_ORANGE_Y,
   LW6MAP_RULES_DEFAULT_START_LIGHTBLUE_Y,
   LW6MAP_RULES_DEFAULT_START_PURPLE_Y,
   LW6MAP_RULES_DEFAULT_START_PINK_Y},
  LW6MAP_RULES_DEFAULT_START_POSITION_MODE,
  LW6MAP_RULES_DEFAULT_COLOR_CONFLICT_MODE,
  LW6MAP_RULES_DEFAULT_SPREAD_THREAD,
  LW6MAP_RULES_DEFAULT_GLUE_POWER,
  LW6MAP_RULES_DEFAULT_BOOST_POWER,
  LW6MAP_RULES_DEFAULT_DANGER_POWER,
  LW6MAP_RULES_DEFAULT_MEDICINE_POWER,
  LW6MAP_RULES_DEFAULT_FRAGS_MODE,
  LW6MAP_RULES_DEFAULT_FRAGS_TO_DISTRIBUTE,
  LW6MAP_RULES_DEFAULT_FRAGS_FADE_OUT,
  LW6MAP_RULES_DEFAULT_USE_TEAM_PROFILES,
  // TEAM_PROFILE_COLOR_AGGRESSIVE
  {
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_RED_AGGRESSIVE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_GREEN_AGGRESSIVE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_BLUE_AGGRESSIVE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_YELLOW_AGGRESSIVE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_CYAN_AGGRESSIVE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_MAGENTA_AGGRESSIVE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_ORANGE_AGGRESSIVE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_LIGHTBLUE_AGGRESSIVE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_PURPLE_AGGRESSIVE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_PINK_AGGRESSIVE},
  // TEAM_PROFILE_COLOR_VULNERABLE
  {
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_RED_VULNERABLE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_GREEN_VULNERABLE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_BLUE_VULNERABLE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_YELLOW_VULNERABLE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_CYAN_VULNERABLE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_MAGENTA_VULNERABLE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_ORANGE_VULNERABLE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_LIGHTBLUE_VULNERABLE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_PURPLE_VULNERABLE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_PINK_VULNERABLE},
  // TEAM_PROFILE_COLOR_MOBILE
  {
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_RED_MOBILE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_GREEN_MOBILE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_BLUE_MOBILE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_YELLOW_MOBILE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_CYAN_MOBILE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_MAGENTA_MOBILE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_ORANGE_MOBILE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_LIGHTBLUE_MOBILE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_PURPLE_MOBILE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_PINK_MOBILE},
  // TEAM_PROFILE_COLOR_FAST
  {
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_RED_FAST,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_GREEN_FAST,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_BLUE_FAST,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_YELLOW_FAST,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_CYAN_FAST,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_MAGENTA_FAST,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_ORANGE_FAST,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_LIGHTBLUE_FAST,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_PURPLE_FAST,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_PINK_FAST},
  // TEAM_PROFILE_COLOR_HANDICAP
  {
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_RED_HANDICAP,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_GREEN_HANDICAP,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_BLUE_HANDICAP,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_YELLOW_HANDICAP,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_CYAN_HANDICAP,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_MAGENTA_HANDICAP,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_ORANGE_HANDICAP,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_LIGHTBLUE_HANDICAP,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_PURPLE_HANDICAP,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_PINK_HANDICAP},
  // TEAM_PROFILE_COLOR_WEAPON_ID
  {
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_RED_WEAPON_ID,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_GREEN_WEAPON_ID,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_BLUE_WEAPON_ID,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_YELLOW_WEAPON_ID,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_CYAN_WEAPON_ID,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_MAGENTA_WEAPON_ID,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_ORANGE_WEAPON_ID,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_LIGHTBLUE_WEAPON_ID,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_PURPLE_WEAPON_ID,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_PINK_WEAPON_ID},
  // TEAM_PROFILE_COLOR_WEAPON_ALTERNATE_ID
  {
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_RED_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_GREEN_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_BLUE_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_YELLOW_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_CYAN_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_MAGENTA_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_ORANGE_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_LIGHTBLUE_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_PURPLE_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_PINK_WEAPON_ALTERNATE_ID},
  // TEAM_PROFILE_COLOR_WEAPON_MODE
  {
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_RED_WEAPON_MODE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_GREEN_WEAPON_MODE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_BLUE_WEAPON_MODE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_YELLOW_WEAPON_MODE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_CYAN_WEAPON_MODE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_MAGENTA_WEAPON_MODE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_ORANGE_WEAPON_MODE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_LIGHTBLUE_WEAPON_MODE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_PURPLE_WEAPON_MODE,
   LW6MAP_RULES_DEFAULT_TEAM_PROFILE_PINK_WEAPON_MODE},
  LW6MAP_RULES_DEFAULT_WEAPON_DURATION,
  LW6MAP_RULES_DEFAULT_WEAPON_CHARGE_DELAY,
  LW6MAP_RULES_DEFAULT_WEAPON_CHARGE_MAX,
  LW6MAP_RULES_DEFAULT_WEAPON_TUNE_BERZERK_POWER,
  LW6MAP_RULES_DEFAULT_WEAPON_TUNE_TURBO_POWER
};

static lw6map_rules_t min_rules = {
  LW6MAP_RULES_MIN_TOTAL_TIME,
  LW6MAP_RULES_MIN_RESPAWN_TEAM,
  LW6MAP_RULES_MIN_RESPAWN_POSITION_MODE,
  LW6MAP_RULES_MIN_RESPAWN_DELAY,
  LW6MAP_RULES_MIN_MOVES_PER_ROUND,
  LW6MAP_RULES_MIN_SPREADS_PER_ROUND,
  LW6MAP_RULES_MIN_ROUNDS_PER_SEC,
  LW6MAP_RULES_MIN_FIGHTER_ATTACK,
  LW6MAP_RULES_MIN_FIGHTER_DEFENSE,
  LW6MAP_RULES_MIN_FIGHTER_NEW_HEALTH,
  LW6MAP_RULES_MIN_FIGHTER_REGENERATE,
  LW6MAP_RULES_MIN_SIDE_ATTACK_FACTOR,
  LW6MAP_RULES_MIN_SIDE_DEFENSE_FACTOR,
  LW6MAP_RULES_MIN_NB_MOVE_TRIES,
  LW6MAP_RULES_MIN_NB_ATTACK_TRIES,
  LW6MAP_RULES_MIN_NB_DEFENSE_TRIES,
  LW6MAP_RULES_MIN_VERTICAL_MOVE,
  LW6MAP_RULES_MIN_SPREAD_MODE,
  LW6MAP_RULES_MIN_SINGLE_ARMY_SIZE,
  LW6MAP_RULES_MIN_TOTAL_ARMIES_SIZE,
  LW6MAP_RULES_MIN_MAX_NB_TEAMS,
  LW6MAP_RULES_MIN_MAX_NB_CURSORS,
  LW6MAP_RULES_MIN_MAX_NB_NODES,
  LW6MAP_RULES_MIN_EXP,
  LW6MAP_RULES_MIN_HIGHEST_TEAM_COLOR_ALLOWED,
  LW6MAP_RULES_MIN_HIGHEST_WEAPON_ALLOWED,
  LW6MAP_RULES_MIN_X_POLARITY,
  LW6MAP_RULES_MIN_Y_POLARITY,
  LW6MAP_RULES_MIN_Z_POLARITY,
  LW6MAP_RULES_MIN_MAX_ZONE_SIZE,
  LW6MAP_RULES_MIN_ROUND_DELTA,
  LW6MAP_RULES_MIN_MAX_ROUND_DELTA,
  LW6MAP_RULES_MIN_MAX_CURSOR_POT,
  LW6MAP_RULES_MIN_CURSOR_POT_INIT,
  LW6MAP_RULES_MIN_MAX_CURSOR_POT_OFFSET,
  // START_COLOR_X
  {
   LW6MAP_RULES_MIN_START_RED_X,
   LW6MAP_RULES_MIN_START_GREEN_X,
   LW6MAP_RULES_MIN_START_BLUE_X,
   LW6MAP_RULES_MIN_START_YELLOW_X,
   LW6MAP_RULES_MIN_START_CYAN_X,
   LW6MAP_RULES_MIN_START_MAGENTA_X,
   LW6MAP_RULES_MIN_START_ORANGE_X,
   LW6MAP_RULES_MIN_START_LIGHTBLUE_X,
   LW6MAP_RULES_MIN_START_PURPLE_X,
   LW6MAP_RULES_MIN_START_PINK_X},
  // START_COLOR_Y
  {
   LW6MAP_RULES_MIN_START_RED_Y,
   LW6MAP_RULES_MIN_START_GREEN_Y,
   LW6MAP_RULES_MIN_START_BLUE_Y,
   LW6MAP_RULES_MIN_START_YELLOW_Y,
   LW6MAP_RULES_MIN_START_CYAN_Y,
   LW6MAP_RULES_MIN_START_MAGENTA_Y,
   LW6MAP_RULES_MIN_START_ORANGE_Y,
   LW6MAP_RULES_MIN_START_LIGHTBLUE_Y,
   LW6MAP_RULES_MIN_START_PURPLE_Y,
   LW6MAP_RULES_MIN_START_PINK_Y},
  LW6MAP_RULES_MIN_START_POSITION_MODE,
  LW6MAP_RULES_MIN_COLOR_CONFLICT_MODE,
  LW6MAP_RULES_MIN_SPREAD_THREAD,
  LW6MAP_RULES_MIN_GLUE_POWER,
  LW6MAP_RULES_MIN_BOOST_POWER,
  LW6MAP_RULES_MIN_DANGER_POWER,
  LW6MAP_RULES_MIN_MEDICINE_POWER,
  LW6MAP_RULES_MIN_FRAGS_MODE,
  LW6MAP_RULES_MIN_FRAGS_TO_DISTRIBUTE,
  LW6MAP_RULES_MIN_FRAGS_FADE_OUT,
  LW6MAP_RULES_MIN_USE_TEAM_PROFILES,
  // TEAM_PROFILE_COLOR_AGGRESSIVE
  {
   LW6MAP_RULES_MIN_TEAM_PROFILE_RED_AGGRESSIVE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_GREEN_AGGRESSIVE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_BLUE_AGGRESSIVE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_YELLOW_AGGRESSIVE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_CYAN_AGGRESSIVE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_MAGENTA_AGGRESSIVE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_ORANGE_AGGRESSIVE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_LIGHTBLUE_AGGRESSIVE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_PURPLE_AGGRESSIVE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_PINK_AGGRESSIVE},
  // TEAM_PROFILE_COLOR_VULNERABLE
  {
   LW6MAP_RULES_MIN_TEAM_PROFILE_RED_VULNERABLE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_GREEN_VULNERABLE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_BLUE_VULNERABLE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_YELLOW_VULNERABLE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_CYAN_VULNERABLE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_MAGENTA_VULNERABLE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_ORANGE_VULNERABLE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_LIGHTBLUE_VULNERABLE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_PURPLE_VULNERABLE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_PINK_VULNERABLE},
  // TEAM_PROFILE_COLOR_MOBILE
  {
   LW6MAP_RULES_MIN_TEAM_PROFILE_RED_MOBILE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_GREEN_MOBILE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_BLUE_MOBILE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_YELLOW_MOBILE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_CYAN_MOBILE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_MAGENTA_MOBILE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_ORANGE_MOBILE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_LIGHTBLUE_MOBILE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_PURPLE_MOBILE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_PINK_MOBILE},
  // TEAM_PROFILE_COLOR_FAST
  {
   LW6MAP_RULES_MIN_TEAM_PROFILE_RED_FAST,
   LW6MAP_RULES_MIN_TEAM_PROFILE_GREEN_FAST,
   LW6MAP_RULES_MIN_TEAM_PROFILE_BLUE_FAST,
   LW6MAP_RULES_MIN_TEAM_PROFILE_YELLOW_FAST,
   LW6MAP_RULES_MIN_TEAM_PROFILE_CYAN_FAST,
   LW6MAP_RULES_MIN_TEAM_PROFILE_MAGENTA_FAST,
   LW6MAP_RULES_MIN_TEAM_PROFILE_ORANGE_FAST,
   LW6MAP_RULES_MIN_TEAM_PROFILE_LIGHTBLUE_FAST,
   LW6MAP_RULES_MIN_TEAM_PROFILE_PURPLE_FAST,
   LW6MAP_RULES_MIN_TEAM_PROFILE_PINK_FAST},
  // TEAM_PROFILE_COLOR_HANDICAP
  {
   LW6MAP_RULES_MIN_TEAM_PROFILE_RED_HANDICAP,
   LW6MAP_RULES_MIN_TEAM_PROFILE_GREEN_HANDICAP,
   LW6MAP_RULES_MIN_TEAM_PROFILE_BLUE_HANDICAP,
   LW6MAP_RULES_MIN_TEAM_PROFILE_YELLOW_HANDICAP,
   LW6MAP_RULES_MIN_TEAM_PROFILE_CYAN_HANDICAP,
   LW6MAP_RULES_MIN_TEAM_PROFILE_MAGENTA_HANDICAP,
   LW6MAP_RULES_MIN_TEAM_PROFILE_ORANGE_HANDICAP,
   LW6MAP_RULES_MIN_TEAM_PROFILE_LIGHTBLUE_HANDICAP,
   LW6MAP_RULES_MIN_TEAM_PROFILE_PURPLE_HANDICAP,
   LW6MAP_RULES_MIN_TEAM_PROFILE_PINK_HANDICAP},
  // TEAM_PROFILE_COLOR_WEAPON_ID
  {
   LW6MAP_RULES_MIN_TEAM_PROFILE_RED_WEAPON_ID,
   LW6MAP_RULES_MIN_TEAM_PROFILE_GREEN_WEAPON_ID,
   LW6MAP_RULES_MIN_TEAM_PROFILE_BLUE_WEAPON_ID,
   LW6MAP_RULES_MIN_TEAM_PROFILE_YELLOW_WEAPON_ID,
   LW6MAP_RULES_MIN_TEAM_PROFILE_CYAN_WEAPON_ID,
   LW6MAP_RULES_MIN_TEAM_PROFILE_MAGENTA_WEAPON_ID,
   LW6MAP_RULES_MIN_TEAM_PROFILE_ORANGE_WEAPON_ID,
   LW6MAP_RULES_MIN_TEAM_PROFILE_LIGHTBLUE_WEAPON_ID,
   LW6MAP_RULES_MIN_TEAM_PROFILE_PURPLE_WEAPON_ID,
   LW6MAP_RULES_MIN_TEAM_PROFILE_PINK_WEAPON_ID},
  // TEAM_PROFILE_COLOR_WEAPON_ALTERNATE_ID
  {
   LW6MAP_RULES_MIN_TEAM_PROFILE_RED_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_MIN_TEAM_PROFILE_GREEN_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_MIN_TEAM_PROFILE_BLUE_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_MIN_TEAM_PROFILE_YELLOW_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_MIN_TEAM_PROFILE_CYAN_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_MIN_TEAM_PROFILE_MAGENTA_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_MIN_TEAM_PROFILE_ORANGE_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_MIN_TEAM_PROFILE_LIGHTBLUE_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_MIN_TEAM_PROFILE_PURPLE_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_MIN_TEAM_PROFILE_PINK_WEAPON_ALTERNATE_ID},
  // TEAM_PROFILE_COLOR_WEAPON_MODE
  {
   LW6MAP_RULES_MIN_TEAM_PROFILE_RED_WEAPON_MODE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_GREEN_WEAPON_MODE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_BLUE_WEAPON_MODE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_YELLOW_WEAPON_MODE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_CYAN_WEAPON_MODE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_MAGENTA_WEAPON_MODE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_ORANGE_WEAPON_MODE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_LIGHTBLUE_WEAPON_MODE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_PURPLE_WEAPON_MODE,
   LW6MAP_RULES_MIN_TEAM_PROFILE_PINK_WEAPON_MODE},
  LW6MAP_RULES_MIN_WEAPON_DURATION,
  LW6MAP_RULES_MIN_WEAPON_CHARGE_DELAY,
  LW6MAP_RULES_MIN_WEAPON_CHARGE_MAX,
  LW6MAP_RULES_MIN_WEAPON_TUNE_BERZERK_POWER,
  LW6MAP_RULES_MIN_WEAPON_TUNE_TURBO_POWER
};

static lw6map_rules_t max_rules = {
  LW6MAP_RULES_MAX_TOTAL_TIME,
  LW6MAP_RULES_MAX_RESPAWN_TEAM,
  LW6MAP_RULES_MAX_RESPAWN_POSITION_MODE,
  LW6MAP_RULES_MAX_RESPAWN_DELAY,
  LW6MAP_RULES_MAX_MOVES_PER_ROUND,
  LW6MAP_RULES_MAX_SPREADS_PER_ROUND,
  LW6MAP_RULES_MAX_ROUNDS_PER_SEC,
  LW6MAP_RULES_MAX_FIGHTER_ATTACK,
  LW6MAP_RULES_MAX_FIGHTER_DEFENSE,
  LW6MAP_RULES_MAX_FIGHTER_NEW_HEALTH,
  LW6MAP_RULES_MAX_FIGHTER_REGENERATE,
  LW6MAP_RULES_MAX_SIDE_ATTACK_FACTOR,
  LW6MAP_RULES_MAX_SIDE_DEFENSE_FACTOR,
  LW6MAP_RULES_MAX_NB_MOVE_TRIES,
  LW6MAP_RULES_MAX_NB_ATTACK_TRIES,
  LW6MAP_RULES_MAX_NB_DEFENSE_TRIES,
  LW6MAP_RULES_MAX_VERTICAL_MOVE,
  LW6MAP_RULES_MAX_SPREAD_MODE,
  LW6MAP_RULES_MAX_SINGLE_ARMY_SIZE,
  LW6MAP_RULES_MAX_TOTAL_ARMIES_SIZE,
  LW6MAP_RULES_MAX_MAX_NB_TEAMS,
  LW6MAP_RULES_MAX_MAX_NB_CURSORS,
  LW6MAP_RULES_MAX_MAX_NB_NODES,
  LW6MAP_RULES_MAX_EXP,
  LW6MAP_RULES_MAX_HIGHEST_TEAM_COLOR_ALLOWED,
  LW6MAP_RULES_MAX_HIGHEST_WEAPON_ALLOWED,
  LW6MAP_RULES_MAX_X_POLARITY,
  LW6MAP_RULES_MAX_Y_POLARITY,
  LW6MAP_RULES_MAX_Z_POLARITY,
  LW6MAP_RULES_MAX_MAX_ZONE_SIZE,
  LW6MAP_RULES_MAX_ROUND_DELTA,
  LW6MAP_RULES_MAX_MAX_ROUND_DELTA,
  LW6MAP_RULES_MAX_MAX_CURSOR_POT,
  LW6MAP_RULES_MAX_CURSOR_POT_INIT,
  LW6MAP_RULES_MAX_MAX_CURSOR_POT_OFFSET,
  // TEAM_PROFILE_START_X
  {
   LW6MAP_RULES_MAX_START_RED_X,
   LW6MAP_RULES_MAX_START_GREEN_X,
   LW6MAP_RULES_MAX_START_BLUE_X,
   LW6MAP_RULES_MAX_START_YELLOW_X,
   LW6MAP_RULES_MAX_START_CYAN_X,
   LW6MAP_RULES_MAX_START_MAGENTA_X,
   LW6MAP_RULES_MAX_START_ORANGE_X,
   LW6MAP_RULES_MAX_START_LIGHTBLUE_X,
   LW6MAP_RULES_MAX_START_PURPLE_X,
   LW6MAP_RULES_MAX_START_PINK_X},
  // TEAM_PROFILE_START_Y
  {
   LW6MAP_RULES_MAX_START_RED_Y,
   LW6MAP_RULES_MAX_START_GREEN_Y,
   LW6MAP_RULES_MAX_START_BLUE_Y,
   LW6MAP_RULES_MAX_START_YELLOW_Y,
   LW6MAP_RULES_MAX_START_CYAN_Y,
   LW6MAP_RULES_MAX_START_MAGENTA_Y,
   LW6MAP_RULES_MAX_START_ORANGE_Y,
   LW6MAP_RULES_MAX_START_LIGHTBLUE_Y,
   LW6MAP_RULES_MAX_START_PURPLE_Y,
   LW6MAP_RULES_MAX_START_PINK_Y},
  LW6MAP_RULES_MAX_START_POSITION_MODE,
  LW6MAP_RULES_MAX_COLOR_CONFLICT_MODE,
  LW6MAP_RULES_MAX_SPREAD_THREAD,
  LW6MAP_RULES_MAX_GLUE_POWER,
  LW6MAP_RULES_MAX_BOOST_POWER,
  LW6MAP_RULES_MAX_DANGER_POWER,
  LW6MAP_RULES_MAX_MEDICINE_POWER,
  LW6MAP_RULES_MAX_FRAGS_MODE,
  LW6MAP_RULES_MAX_FRAGS_TO_DISTRIBUTE,
  LW6MAP_RULES_MAX_FRAGS_FADE_OUT,
  LW6MAP_RULES_MAX_USE_TEAM_PROFILES,
  // TEAM_PROFILE_COLOR_AGGRESSIVE
  {
   LW6MAP_RULES_MAX_TEAM_PROFILE_RED_AGGRESSIVE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_GREEN_AGGRESSIVE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_BLUE_AGGRESSIVE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_YELLOW_AGGRESSIVE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_CYAN_AGGRESSIVE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_MAGENTA_AGGRESSIVE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_ORANGE_AGGRESSIVE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_LIGHTBLUE_AGGRESSIVE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_PURPLE_AGGRESSIVE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_PINK_AGGRESSIVE},
  // TEAM_PROFILE_COLOR_VULNERABLE
  {
   LW6MAP_RULES_MAX_TEAM_PROFILE_RED_VULNERABLE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_GREEN_VULNERABLE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_BLUE_VULNERABLE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_YELLOW_VULNERABLE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_CYAN_VULNERABLE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_MAGENTA_VULNERABLE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_ORANGE_VULNERABLE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_LIGHTBLUE_VULNERABLE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_PURPLE_VULNERABLE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_PINK_VULNERABLE},
  // TEAM_PROFILE_COLOR_VULNERABLE
  {
   LW6MAP_RULES_MAX_TEAM_PROFILE_RED_MOBILE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_GREEN_MOBILE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_BLUE_MOBILE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_YELLOW_MOBILE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_CYAN_MOBILE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_MAGENTA_MOBILE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_ORANGE_MOBILE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_LIGHTBLUE_MOBILE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_PURPLE_MOBILE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_PINK_MOBILE},
  // TEAM_PROFILE_COLOR_FAST
  {
   LW6MAP_RULES_MAX_TEAM_PROFILE_RED_FAST,
   LW6MAP_RULES_MAX_TEAM_PROFILE_GREEN_FAST,
   LW6MAP_RULES_MAX_TEAM_PROFILE_BLUE_FAST,
   LW6MAP_RULES_MAX_TEAM_PROFILE_YELLOW_FAST,
   LW6MAP_RULES_MAX_TEAM_PROFILE_CYAN_FAST,
   LW6MAP_RULES_MAX_TEAM_PROFILE_MAGENTA_FAST,
   LW6MAP_RULES_MAX_TEAM_PROFILE_ORANGE_FAST,
   LW6MAP_RULES_MAX_TEAM_PROFILE_LIGHTBLUE_FAST,
   LW6MAP_RULES_MAX_TEAM_PROFILE_PURPLE_FAST,
   LW6MAP_RULES_MAX_TEAM_PROFILE_PINK_FAST},
  // TEAM_PROFILE_COLOR_HANDICAP
  {
   LW6MAP_RULES_MAX_TEAM_PROFILE_RED_HANDICAP,
   LW6MAP_RULES_MAX_TEAM_PROFILE_GREEN_HANDICAP,
   LW6MAP_RULES_MAX_TEAM_PROFILE_BLUE_HANDICAP,
   LW6MAP_RULES_MAX_TEAM_PROFILE_YELLOW_HANDICAP,
   LW6MAP_RULES_MAX_TEAM_PROFILE_CYAN_HANDICAP,
   LW6MAP_RULES_MAX_TEAM_PROFILE_MAGENTA_HANDICAP,
   LW6MAP_RULES_MAX_TEAM_PROFILE_ORANGE_HANDICAP,
   LW6MAP_RULES_MAX_TEAM_PROFILE_LIGHTBLUE_HANDICAP,
   LW6MAP_RULES_MAX_TEAM_PROFILE_PURPLE_HANDICAP,
   LW6MAP_RULES_MAX_TEAM_PROFILE_PINK_HANDICAP},
  // TEAM_PROFILE_COLOR_WEAPON_ID
  {
   LW6MAP_RULES_MAX_TEAM_PROFILE_RED_WEAPON_ID,
   LW6MAP_RULES_MAX_TEAM_PROFILE_GREEN_WEAPON_ID,
   LW6MAP_RULES_MAX_TEAM_PROFILE_BLUE_WEAPON_ID,
   LW6MAP_RULES_MAX_TEAM_PROFILE_YELLOW_WEAPON_ID,
   LW6MAP_RULES_MAX_TEAM_PROFILE_CYAN_WEAPON_ID,
   LW6MAP_RULES_MAX_TEAM_PROFILE_MAGENTA_WEAPON_ID,
   LW6MAP_RULES_MAX_TEAM_PROFILE_ORANGE_WEAPON_ID,
   LW6MAP_RULES_MAX_TEAM_PROFILE_LIGHTBLUE_WEAPON_ID,
   LW6MAP_RULES_MAX_TEAM_PROFILE_PURPLE_WEAPON_ID,
   LW6MAP_RULES_MAX_TEAM_PROFILE_PINK_WEAPON_ID},
  // TEAM_PROFILE_COLOR_WEAPON_ALTERNATE_ID
  {
   LW6MAP_RULES_MAX_TEAM_PROFILE_RED_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_MAX_TEAM_PROFILE_GREEN_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_MAX_TEAM_PROFILE_BLUE_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_MAX_TEAM_PROFILE_YELLOW_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_MAX_TEAM_PROFILE_CYAN_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_MAX_TEAM_PROFILE_MAGENTA_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_MAX_TEAM_PROFILE_ORANGE_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_MAX_TEAM_PROFILE_LIGHTBLUE_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_MAX_TEAM_PROFILE_PURPLE_WEAPON_ALTERNATE_ID,
   LW6MAP_RULES_MAX_TEAM_PROFILE_PINK_WEAPON_ALTERNATE_ID},
  // TEAM_PROFILE_COLOR_WEAPON_MODE
  {
   LW6MAP_RULES_MAX_TEAM_PROFILE_RED_WEAPON_MODE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_GREEN_WEAPON_MODE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_BLUE_WEAPON_MODE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_YELLOW_WEAPON_MODE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_CYAN_WEAPON_MODE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_MAGENTA_WEAPON_MODE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_ORANGE_WEAPON_MODE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_LIGHTBLUE_WEAPON_MODE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_PURPLE_WEAPON_MODE,
   LW6MAP_RULES_MAX_TEAM_PROFILE_PINK_WEAPON_MODE},
  LW6MAP_RULES_MAX_WEAPON_DURATION,
  LW6MAP_RULES_MAX_WEAPON_CHARGE_DELAY,
  LW6MAP_RULES_MAX_WEAPON_CHARGE_MAX,
  LW6MAP_RULES_MAX_WEAPON_TUNE_BERZERK_POWER,
  LW6MAP_RULES_MAX_WEAPON_TUNE_TURBO_POWER
};

/**
 * lw6map_rules_zero
 *
 * @sys_context: global system context
 * @rules: struct to initialize
 *
 * Sets a rules struct to zero, simply puts zero everywhere
 * without checking what was here before
 *
 * Return value: none.
 */
void
lw6map_rules_zero (lw6sys_context_t * sys_context, lw6map_rules_t * rules)
{
  memset (rules, 0, sizeof (lw6map_rules_t));
}

/**
 * lw6map_rules_defaults
 *
 * @sys_context: global system context
 * @rules: struct to set to defaults
 *
 * Set rules to default values, as these are all integers,
 * you can call this on any rules object.
 *
 * Return value: none.
 */
void
lw6map_rules_defaults (lw6sys_context_t * sys_context, lw6map_rules_t * rules)
{
  lw6map_rules_copy (sys_context, rules, &default_rules);
}

/**
 * lw6map_rules_copy
 *
 * @sys_context: global system context
 * @dst: destination (out param)
 * @src: source (in param)
 *
 * Copies the data from source to destination, simple
 * wrapper on memcpy.
 *
 * Return value: none.
 */
void
lw6map_rules_copy (lw6sys_context_t * sys_context, lw6map_rules_t * dst, const lw6map_rules_t * src)
{
  /*
   * Note that in this context, we'd better not use strings
   * in this structure, or code would become a bit more complex.
   */
  memcpy (dst, src, sizeof (lw6map_rules_t));
}

/**
 * lw6map_rules_update_checksum
 *
 * @sys_context: global system context
 * @rules: rules struct to check
 * @checksum: checksum to update (in/out param)
 *
 * Updates a checksum with the rules data.
 *
 * Return value: none.
 */
void
lw6map_rules_update_checksum (lw6sys_context_t * sys_context, const lw6map_rules_t * rules, u_int32_t * checksum)
{
  int i = 0;

  lw6sys_checksum_update_int32 (sys_context, checksum, rules->total_time);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->respawn_team);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->respawn_position_mode);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->respawn_delay);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->moves_per_round);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->spreads_per_round);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->rounds_per_sec);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->fighter_attack);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->fighter_defense);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->fighter_new_health);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->fighter_regenerate);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->side_attack_factor);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->side_defense_factor);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->nb_move_tries);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->nb_attack_tries);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->nb_defense_tries);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->vertical_move);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->spread_mode);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->single_army_size);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->total_armies_size);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->max_nb_teams);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->max_nb_cursors);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->max_nb_nodes);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->exp);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->highest_team_color_allowed);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->highest_weapon_allowed);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->x_polarity);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->y_polarity);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->z_polarity);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->max_zone_size);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->round_delta);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->max_round_delta);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->max_cursor_pot);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->cursor_pot_init);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->max_cursor_pot_offset);
  for (i = 0; i < LW6MAP_MAX_NB_TEAMS; ++i)
    {
      lw6sys_checksum_update_int32 (sys_context, checksum, rules->start_x[i]);
    }
  for (i = 0; i < LW6MAP_MAX_NB_TEAMS; ++i)
    {
      lw6sys_checksum_update_int32 (sys_context, checksum, rules->start_y[i]);
    }
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->start_position_mode);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->color_conflict_mode);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->spread_thread);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->glue_power);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->boost_power);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->danger_power);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->medicine_power);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->frags_mode);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->frags_to_distribute);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->frags_fade_out);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->use_team_profiles);
  for (i = 0; i < LW6MAP_MAX_NB_TEAMS; ++i)
    {
      lw6sys_checksum_update_int32 (sys_context, checksum, rules->team_profile_aggressive[i]);
    }
  for (i = 0; i < LW6MAP_MAX_NB_TEAMS; ++i)
    {
      lw6sys_checksum_update_int32 (sys_context, checksum, rules->team_profile_vulnerable[i]);
    }
  for (i = 0; i < LW6MAP_MAX_NB_TEAMS; ++i)
    {
      lw6sys_checksum_update_int32 (sys_context, checksum, rules->team_profile_mobile[i]);
    }
  for (i = 0; i < LW6MAP_MAX_NB_TEAMS; ++i)
    {
      lw6sys_checksum_update_int32 (sys_context, checksum, rules->team_profile_fast[i]);
    }
  for (i = 0; i < LW6MAP_MAX_NB_TEAMS; ++i)
    {
      lw6sys_checksum_update_int32 (sys_context, checksum, rules->team_profile_handicap[i]);
    }
  for (i = 0; i < LW6MAP_MAX_NB_TEAMS; ++i)
    {
      lw6sys_checksum_update_int32 (sys_context, checksum, rules->team_profile_weapon_id[i]);
    }
  for (i = 0; i < LW6MAP_MAX_NB_TEAMS; ++i)
    {
      lw6sys_checksum_update_int32 (sys_context, checksum, rules->team_profile_weapon_alternate_id[i]);
    }
  for (i = 0; i < LW6MAP_MAX_NB_TEAMS; ++i)
    {
      lw6sys_checksum_update_int32 (sys_context, checksum, rules->team_profile_weapon_mode[i]);
    }
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->weapon_duration);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->weapon_charge_delay);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->weapon_charge_max);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->weapon_tune_berzerk_power);
  lw6sys_checksum_update_int32 (sys_context, checksum, rules->weapon_tune_turbo_power);
}

static const int32_t *
_get_rules_int_ptr (lw6sys_context_t * sys_context, const lw6map_rules_t * rules, const char *key)
{
  const int32_t *ret = NULL;
  char *formatted_key = NULL;

  if (rules && key)
    {
      formatted_key = lw6sys_keyword_as_key (sys_context, key);
      if (formatted_key)
	{
	  if (!strcmp (formatted_key, LW6DEF_TOTAL_TIME))
	    {
	      ret = &(rules->total_time);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_RESPAWN_TEAM))
	    {
	      ret = &(rules->respawn_team);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_RESPAWN_POSITION_MODE))
	    {
	      ret = &(rules->respawn_position_mode);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_RESPAWN_DELAY))
	    {
	      ret = &(rules->respawn_delay);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_MOVES_PER_ROUND))
	    {
	      ret = &(rules->moves_per_round);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_SPREADS_PER_ROUND))
	    {
	      ret = &(rules->spreads_per_round);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_ROUNDS_PER_SEC))
	    {
	      ret = &(rules->rounds_per_sec);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_FIGHTER_ATTACK))
	    {
	      ret = &(rules->fighter_attack);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_FIGHTER_DEFENSE))
	    {
	      ret = &(rules->fighter_defense);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_FIGHTER_NEW_HEALTH))
	    {
	      ret = &(rules->fighter_new_health);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_FIGHTER_REGENERATE))
	    {
	      ret = &(rules->fighter_regenerate);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_SIDE_ATTACK_FACTOR))
	    {
	      ret = &(rules->side_attack_factor);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_SIDE_DEFENSE_FACTOR))
	    {
	      ret = &(rules->side_defense_factor);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_NB_MOVE_TRIES))
	    {
	      ret = &(rules->nb_move_tries);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_NB_ATTACK_TRIES))
	    {
	      ret = &(rules->nb_attack_tries);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_NB_DEFENSE_TRIES))
	    {
	      ret = &(rules->nb_defense_tries);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_VERTICAL_MOVE))
	    {
	      ret = &(rules->vertical_move);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_SPREAD_MODE))
	    {
	      ret = &(rules->spread_mode);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_SINGLE_ARMY_SIZE))
	    {
	      ret = &(rules->single_army_size);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TOTAL_ARMIES_SIZE))
	    {
	      ret = &(rules->total_armies_size);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_MAX_NB_TEAMS))
	    {
	      ret = &(rules->max_nb_teams);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_MAX_NB_CURSORS))
	    {
	      ret = &(rules->max_nb_cursors);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_MAX_NB_NODES))
	    {
	      ret = &(rules->max_nb_nodes);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_EXP))
	    {
	      ret = &(rules->exp);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_HIGHEST_TEAM_COLOR_ALLOWED))
	    {
	      ret = &(rules->highest_team_color_allowed);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_HIGHEST_WEAPON_ALLOWED))
	    {
	      ret = &(rules->highest_weapon_allowed);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_X_POLARITY))
	    {
	      ret = &(rules->x_polarity);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_Y_POLARITY))
	    {
	      ret = &(rules->y_polarity);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_Z_POLARITY))
	    {
	      ret = &(rules->z_polarity);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_MAX_ZONE_SIZE))
	    {
	      ret = &(rules->max_zone_size);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_ROUND_DELTA))
	    {
	      ret = &(rules->round_delta);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_MAX_ROUND_DELTA))
	    {
	      ret = &(rules->max_round_delta);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_MAX_CURSOR_POT))
	    {
	      ret = &(rules->max_cursor_pot);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_CURSOR_POT_INIT))
	    {
	      ret = &(rules->cursor_pot_init);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_MAX_CURSOR_POT_OFFSET))
	    {
	      ret = &(rules->max_cursor_pot_offset);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_RED_X))
	    {
	      ret = &(rules->start_x[LW6MAP_TEAM_COLOR_RED]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_GREEN_X))
	    {
	      ret = &(rules->start_x[LW6MAP_TEAM_COLOR_GREEN]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_BLUE_X))
	    {
	      ret = &(rules->start_x[LW6MAP_TEAM_COLOR_BLUE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_YELLOW_X))
	    {
	      ret = &(rules->start_x[LW6MAP_TEAM_COLOR_YELLOW]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_CYAN_X))
	    {
	      ret = &(rules->start_x[LW6MAP_TEAM_COLOR_CYAN]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_MAGENTA_X))
	    {
	      ret = &(rules->start_x[LW6MAP_TEAM_COLOR_MAGENTA]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_ORANGE_X))
	    {
	      ret = &(rules->start_x[LW6MAP_TEAM_COLOR_ORANGE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_LIGHTBLUE_X))
	    {
	      ret = &(rules->start_x[LW6MAP_TEAM_COLOR_LIGHTBLUE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_PURPLE_X))
	    {
	      ret = &(rules->start_x[LW6MAP_TEAM_COLOR_PURPLE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_PINK_X))
	    {
	      ret = &(rules->start_x[LW6MAP_TEAM_COLOR_PINK]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_RED_Y))
	    {
	      ret = &(rules->start_y[LW6MAP_TEAM_COLOR_RED]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_GREEN_Y))
	    {
	      ret = &(rules->start_y[LW6MAP_TEAM_COLOR_GREEN]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_BLUE_Y))
	    {
	      ret = &(rules->start_y[LW6MAP_TEAM_COLOR_BLUE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_YELLOW_Y))
	    {
	      ret = &(rules->start_y[LW6MAP_TEAM_COLOR_YELLOW]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_CYAN_Y))
	    {
	      ret = &(rules->start_y[LW6MAP_TEAM_COLOR_CYAN]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_MAGENTA_Y))
	    {
	      ret = &(rules->start_y[LW6MAP_TEAM_COLOR_MAGENTA]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_ORANGE_Y))
	    {
	      ret = &(rules->start_y[LW6MAP_TEAM_COLOR_ORANGE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_LIGHTBLUE_Y))
	    {
	      ret = &(rules->start_y[LW6MAP_TEAM_COLOR_LIGHTBLUE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_PURPLE_Y))
	    {
	      ret = &(rules->start_y[LW6MAP_TEAM_COLOR_PURPLE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_PINK_Y))
	    {
	      ret = &(rules->start_y[LW6MAP_TEAM_COLOR_PINK]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_START_POSITION_MODE))
	    {
	      ret = &(rules->start_position_mode);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_COLOR_CONFLICT_MODE))
	    {
	      ret = &(rules->color_conflict_mode);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_SPREAD_THREAD))
	    {
	      ret = &(rules->spread_thread);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_GLUE_POWER))
	    {
	      ret = &(rules->glue_power);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_BOOST_POWER))
	    {
	      ret = &(rules->boost_power);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_DANGER_POWER))
	    {
	      ret = &(rules->danger_power);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_MEDICINE_POWER))
	    {
	      ret = &(rules->medicine_power);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_FRAGS_MODE))
	    {
	      ret = &(rules->frags_mode);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_FRAGS_TO_DISTRIBUTE))
	    {
	      ret = &(rules->frags_to_distribute);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_FRAGS_FADE_OUT))
	    {
	      ret = &(rules->frags_fade_out);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_USE_TEAM_PROFILES))
	    {
	      ret = &(rules->use_team_profiles);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_RED_AGGRESSIVE))
	    {
	      ret = &(rules->team_profile_aggressive[LW6MAP_TEAM_COLOR_RED]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_GREEN_AGGRESSIVE))
	    {
	      ret = &(rules->team_profile_aggressive[LW6MAP_TEAM_COLOR_GREEN]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_BLUE_AGGRESSIVE))
	    {
	      ret = &(rules->team_profile_aggressive[LW6MAP_TEAM_COLOR_BLUE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_YELLOW_AGGRESSIVE))
	    {
	      ret = &(rules->team_profile_aggressive[LW6MAP_TEAM_COLOR_YELLOW]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_CYAN_AGGRESSIVE))
	    {
	      ret = &(rules->team_profile_aggressive[LW6MAP_TEAM_COLOR_CYAN]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_MAGENTA_AGGRESSIVE))
	    {
	      ret = &(rules->team_profile_aggressive[LW6MAP_TEAM_COLOR_MAGENTA]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_ORANGE_AGGRESSIVE))
	    {
	      ret = &(rules->team_profile_aggressive[LW6MAP_TEAM_COLOR_ORANGE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_LIGHTBLUE_AGGRESSIVE))
	    {
	      ret = &(rules->team_profile_aggressive[LW6MAP_TEAM_COLOR_LIGHTBLUE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_PURPLE_AGGRESSIVE))
	    {
	      ret = &(rules->team_profile_aggressive[LW6MAP_TEAM_COLOR_PURPLE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_PINK_AGGRESSIVE))
	    {
	      ret = &(rules->team_profile_aggressive[LW6MAP_TEAM_COLOR_PINK]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_RED_VULNERABLE))
	    {
	      ret = &(rules->team_profile_vulnerable[LW6MAP_TEAM_COLOR_RED]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_GREEN_VULNERABLE))
	    {
	      ret = &(rules->team_profile_vulnerable[LW6MAP_TEAM_COLOR_GREEN]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_BLUE_VULNERABLE))
	    {
	      ret = &(rules->team_profile_vulnerable[LW6MAP_TEAM_COLOR_BLUE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_YELLOW_VULNERABLE))
	    {
	      ret = &(rules->team_profile_vulnerable[LW6MAP_TEAM_COLOR_YELLOW]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_CYAN_VULNERABLE))
	    {
	      ret = &(rules->team_profile_vulnerable[LW6MAP_TEAM_COLOR_CYAN]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_MAGENTA_VULNERABLE))
	    {
	      ret = &(rules->team_profile_vulnerable[LW6MAP_TEAM_COLOR_MAGENTA]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_ORANGE_VULNERABLE))
	    {
	      ret = &(rules->team_profile_vulnerable[LW6MAP_TEAM_COLOR_ORANGE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_LIGHTBLUE_VULNERABLE))
	    {
	      ret = &(rules->team_profile_vulnerable[LW6MAP_TEAM_COLOR_LIGHTBLUE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_PURPLE_VULNERABLE))
	    {
	      ret = &(rules->team_profile_vulnerable[LW6MAP_TEAM_COLOR_PURPLE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_PINK_VULNERABLE))
	    {
	      ret = &(rules->team_profile_vulnerable[LW6MAP_TEAM_COLOR_PINK]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_RED_MOBILE))
	    {
	      ret = &(rules->team_profile_mobile[LW6MAP_TEAM_COLOR_RED]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_GREEN_MOBILE))
	    {
	      ret = &(rules->team_profile_mobile[LW6MAP_TEAM_COLOR_GREEN]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_BLUE_MOBILE))
	    {
	      ret = &(rules->team_profile_mobile[LW6MAP_TEAM_COLOR_BLUE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_YELLOW_MOBILE))
	    {
	      ret = &(rules->team_profile_mobile[LW6MAP_TEAM_COLOR_YELLOW]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_CYAN_MOBILE))
	    {
	      ret = &(rules->team_profile_mobile[LW6MAP_TEAM_COLOR_CYAN]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_MAGENTA_MOBILE))
	    {
	      ret = &(rules->team_profile_mobile[LW6MAP_TEAM_COLOR_MAGENTA]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_ORANGE_MOBILE))
	    {
	      ret = &(rules->team_profile_mobile[LW6MAP_TEAM_COLOR_ORANGE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_LIGHTBLUE_MOBILE))
	    {
	      ret = &(rules->team_profile_mobile[LW6MAP_TEAM_COLOR_LIGHTBLUE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_PURPLE_MOBILE))
	    {
	      ret = &(rules->team_profile_mobile[LW6MAP_TEAM_COLOR_PURPLE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_PINK_MOBILE))
	    {
	      ret = &(rules->team_profile_mobile[LW6MAP_TEAM_COLOR_PINK]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_RED_FAST))
	    {
	      ret = &(rules->team_profile_fast[LW6MAP_TEAM_COLOR_RED]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_GREEN_FAST))
	    {
	      ret = &(rules->team_profile_fast[LW6MAP_TEAM_COLOR_GREEN]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_BLUE_FAST))
	    {
	      ret = &(rules->team_profile_fast[LW6MAP_TEAM_COLOR_BLUE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_YELLOW_FAST))
	    {
	      ret = &(rules->team_profile_fast[LW6MAP_TEAM_COLOR_YELLOW]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_CYAN_FAST))
	    {
	      ret = &(rules->team_profile_fast[LW6MAP_TEAM_COLOR_CYAN]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_MAGENTA_FAST))
	    {
	      ret = &(rules->team_profile_fast[LW6MAP_TEAM_COLOR_MAGENTA]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_ORANGE_FAST))
	    {
	      ret = &(rules->team_profile_fast[LW6MAP_TEAM_COLOR_ORANGE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_LIGHTBLUE_FAST))
	    {
	      ret = &(rules->team_profile_fast[LW6MAP_TEAM_COLOR_LIGHTBLUE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_PURPLE_FAST))
	    {
	      ret = &(rules->team_profile_fast[LW6MAP_TEAM_COLOR_PURPLE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_PINK_FAST))
	    {
	      ret = &(rules->team_profile_fast[LW6MAP_TEAM_COLOR_PINK]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_RED_HANDICAP))
	    {
	      ret = &(rules->team_profile_handicap[LW6MAP_TEAM_COLOR_RED]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_GREEN_HANDICAP))
	    {
	      ret = &(rules->team_profile_handicap[LW6MAP_TEAM_COLOR_GREEN]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_BLUE_HANDICAP))
	    {
	      ret = &(rules->team_profile_handicap[LW6MAP_TEAM_COLOR_BLUE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_YELLOW_HANDICAP))
	    {
	      ret = &(rules->team_profile_handicap[LW6MAP_TEAM_COLOR_YELLOW]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_CYAN_HANDICAP))
	    {
	      ret = &(rules->team_profile_handicap[LW6MAP_TEAM_COLOR_CYAN]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_MAGENTA_HANDICAP))
	    {
	      ret = &(rules->team_profile_handicap[LW6MAP_TEAM_COLOR_MAGENTA]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_ORANGE_HANDICAP))
	    {
	      ret = &(rules->team_profile_handicap[LW6MAP_TEAM_COLOR_ORANGE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_LIGHTBLUE_HANDICAP))
	    {
	      ret = &(rules->team_profile_handicap[LW6MAP_TEAM_COLOR_LIGHTBLUE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_PURPLE_HANDICAP))
	    {
	      ret = &(rules->team_profile_handicap[LW6MAP_TEAM_COLOR_PURPLE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_PINK_HANDICAP))
	    {
	      ret = &(rules->team_profile_handicap[LW6MAP_TEAM_COLOR_PINK]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_RED_WEAPON_ID))
	    {
	      ret = &(rules->team_profile_weapon_id[LW6MAP_TEAM_COLOR_RED]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_GREEN_WEAPON_ID))
	    {
	      ret = &(rules->team_profile_weapon_id[LW6MAP_TEAM_COLOR_GREEN]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_BLUE_WEAPON_ID))
	    {
	      ret = &(rules->team_profile_weapon_id[LW6MAP_TEAM_COLOR_BLUE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_YELLOW_WEAPON_ID))
	    {
	      ret = &(rules->team_profile_weapon_id[LW6MAP_TEAM_COLOR_YELLOW]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_CYAN_WEAPON_ID))
	    {
	      ret = &(rules->team_profile_weapon_id[LW6MAP_TEAM_COLOR_CYAN]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_MAGENTA_WEAPON_ID))
	    {
	      ret = &(rules->team_profile_weapon_id[LW6MAP_TEAM_COLOR_MAGENTA]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_ORANGE_WEAPON_ID))
	    {
	      ret = &(rules->team_profile_weapon_id[LW6MAP_TEAM_COLOR_ORANGE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_LIGHTBLUE_WEAPON_ID))
	    {
	      ret = &(rules->team_profile_weapon_id[LW6MAP_TEAM_COLOR_LIGHTBLUE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_PURPLE_WEAPON_ID))
	    {
	      ret = &(rules->team_profile_weapon_id[LW6MAP_TEAM_COLOR_PURPLE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_PINK_WEAPON_ID))
	    {
	      ret = &(rules->team_profile_weapon_id[LW6MAP_TEAM_COLOR_PINK]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_RED_WEAPON_ALTERNATE_ID))
	    {
	      ret = &(rules->team_profile_weapon_alternate_id[LW6MAP_TEAM_COLOR_RED]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_GREEN_WEAPON_ALTERNATE_ID))
	    {
	      ret = &(rules->team_profile_weapon_alternate_id[LW6MAP_TEAM_COLOR_GREEN]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_BLUE_WEAPON_ALTERNATE_ID))
	    {
	      ret = &(rules->team_profile_weapon_alternate_id[LW6MAP_TEAM_COLOR_BLUE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_YELLOW_WEAPON_ALTERNATE_ID))
	    {
	      ret = &(rules->team_profile_weapon_alternate_id[LW6MAP_TEAM_COLOR_YELLOW]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_CYAN_WEAPON_ALTERNATE_ID))
	    {
	      ret = &(rules->team_profile_weapon_alternate_id[LW6MAP_TEAM_COLOR_CYAN]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_MAGENTA_WEAPON_ALTERNATE_ID))
	    {
	      ret = &(rules->team_profile_weapon_alternate_id[LW6MAP_TEAM_COLOR_MAGENTA]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_ORANGE_WEAPON_ALTERNATE_ID))
	    {
	      ret = &(rules->team_profile_weapon_alternate_id[LW6MAP_TEAM_COLOR_ORANGE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_LIGHTBLUE_WEAPON_ALTERNATE_ID))
	    {
	      ret = &(rules->team_profile_weapon_alternate_id[LW6MAP_TEAM_COLOR_LIGHTBLUE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_PURPLE_WEAPON_ALTERNATE_ID))
	    {
	      ret = &(rules->team_profile_weapon_alternate_id[LW6MAP_TEAM_COLOR_PURPLE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_PINK_WEAPON_ALTERNATE_ID))
	    {
	      ret = &(rules->team_profile_weapon_alternate_id[LW6MAP_TEAM_COLOR_PINK]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_RED_WEAPON_MODE))
	    {
	      ret = &(rules->team_profile_weapon_mode[LW6MAP_TEAM_COLOR_RED]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_GREEN_WEAPON_MODE))
	    {
	      ret = &(rules->team_profile_weapon_mode[LW6MAP_TEAM_COLOR_GREEN]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_BLUE_WEAPON_MODE))
	    {
	      ret = &(rules->team_profile_weapon_mode[LW6MAP_TEAM_COLOR_BLUE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_YELLOW_WEAPON_MODE))
	    {
	      ret = &(rules->team_profile_weapon_mode[LW6MAP_TEAM_COLOR_YELLOW]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_CYAN_WEAPON_MODE))
	    {
	      ret = &(rules->team_profile_weapon_mode[LW6MAP_TEAM_COLOR_CYAN]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_MAGENTA_WEAPON_MODE))
	    {
	      ret = &(rules->team_profile_weapon_mode[LW6MAP_TEAM_COLOR_MAGENTA]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_ORANGE_WEAPON_MODE))
	    {
	      ret = &(rules->team_profile_weapon_mode[LW6MAP_TEAM_COLOR_ORANGE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_LIGHTBLUE_WEAPON_MODE))
	    {
	      ret = &(rules->team_profile_weapon_mode[LW6MAP_TEAM_COLOR_LIGHTBLUE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_PURPLE_WEAPON_MODE))
	    {
	      ret = &(rules->team_profile_weapon_mode[LW6MAP_TEAM_COLOR_PURPLE]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_TEAM_PROFILE_PINK_WEAPON_MODE))
	    {
	      ret = &(rules->team_profile_weapon_mode[LW6MAP_TEAM_COLOR_PINK]);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_WEAPON_DURATION))
	    {
	      ret = &(rules->weapon_duration);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_WEAPON_CHARGE_DELAY))
	    {
	      ret = &(rules->weapon_charge_delay);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_WEAPON_CHARGE_MAX))
	    {
	      ret = &(rules->weapon_charge_max);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_WEAPON_TUNE_BERZERK_POWER))
	    {
	      ret = &(rules->weapon_tune_berzerk_power);
	    }
	  else if (!strcmp (formatted_key, LW6DEF_WEAPON_TUNE_TURBO_POWER))
	    {
	      ret = &(rules->weapon_tune_turbo_power);
	    }
	  else
	    {
	      lw6sys_log (sys_context, LW6SYS_LOG_WARNING, _x_ ("static rules key \"%s\" does not exist, can't get a pointer on its value"), key);
	    }
	  LW6SYS_FREE (sys_context, formatted_key);
	}
    }

  return ret;
}

/**
 * lw6map_rules_get_default
 *
 * @sys_context: global system context
 * @key: key to query
 *
 * Get the default value for a given string key. Of course
 * you could access the member, but this function internally does
 * the conversion between readable string and actual
 * struct offset.
 *
 * Return value: integer.
 */
int32_t
lw6map_rules_get_default (lw6sys_context_t * sys_context, const char *key)
{
  const int32_t *ptr;
  int32_t ret = 0;

  ptr = _get_rules_int_ptr (sys_context, &default_rules, key);
  if (ptr)
    {
      ret = (*ptr);
    }

  return ret;
}

/**
 * lw6map_rules_get_min
 *
 * @sys_context: global system context
 * @key: key to query
 *
 * Get the min value for a given string key. Of course
 * you could access the member, but this function internally does
 * the conversion between readable string and actual
 * struct offset.
 *
 * Return value: integer.
 */
int32_t
lw6map_rules_get_min (lw6sys_context_t * sys_context, const char *key)
{
  const int32_t *ptr;
  int32_t ret = 0;

  ptr = _get_rules_int_ptr (sys_context, &min_rules, key);
  if (ptr)
    {
      ret = (*ptr);
    }

  return ret;
}

/**
 * lw6map_rules_get_min
 *
 * @sys_context: global system context
 * @key: key to query
 *
 * Get the min value for a given string key. Of course
 * you could access the member, but this function internally does
 * the conversion between readable string and actual
 * struct offset.
 *
 * Return value: integer.
 */
int32_t
lw6map_rules_get_max (lw6sys_context_t * sys_context, const char *key)
{
  const int32_t *ptr;
  int32_t ret = 0;

  ptr = _get_rules_int_ptr (sys_context, &max_rules, key);
  if (ptr)
    {
      ret = (*ptr);
    }

  return ret;
}

/**
 * lw6map_rules_get_int
 *
 * @sys_context: global system context
 * @rules: struct to use
 * @key: key to query
 *
 * Get the value for a given string key, as an integer. Of course
 * you could access the member, but this function internally does
 * the conversion between readable string and actual
 * struct offset.
 *
 * Return value: integer.
 */
int32_t
lw6map_rules_get_int (lw6sys_context_t * sys_context, const lw6map_rules_t * rules, const char *key)
{
  int32_t ret = 0;
  const int32_t *ptr;
  int32_t min_value;
  int32_t max_value;

  ptr = _get_rules_int_ptr (sys_context, rules, key);
  if (ptr)
    {
      min_value = lw6map_rules_get_min (sys_context, key);
      max_value = lw6map_rules_get_max (sys_context, key);
      if ((*ptr) < min_value)
	{
	  lw6sys_log (sys_context, LW6SYS_LOG_WARNING,
		      _x_ ("can't get static option \"%s\" (%d), it's too small, returning min value %d instead"), key, (*ptr), min_value);
	  ret = min_value;
	}
      else if ((*ptr) > max_value)
	{
	  lw6sys_log (sys_context, LW6SYS_LOG_WARNING,
		      _x_ ("can't get static option \"%s\" (%d), it's too big, returning max value %d instead"), key, (*ptr), max_value);
	  ret = max_value;
	}
      else
	{
	  ret = (*ptr);
	}
    }

  return ret;
}

/**
 * lw6map_rules_set_int
 *
 * @sys_context: global system context
 * @rules: struct to use
 * @key: key to set
 * @value: new integer value for key
 *
 * Set the value for a given string key, as an integer. Of course
 * you could access the member, but this function internally does
 * the conversion between readable string and actual
 * struct offset.
 *
 * Return value: 1 on success, 0 on failure (eg key not found)
 */
int
lw6map_rules_set_int (lw6sys_context_t * sys_context, lw6map_rules_t * rules, const char *key, int32_t value)
{
  int32_t *ptr;
  int32_t min_value;
  int32_t max_value;
  int ret = 0;

  /*
   * We cast const to not const, actually in that case it's OK,
   * the rules we used are really supposed to be writeable.
   */
  ptr = (int32_t *) _get_rules_int_ptr (sys_context, rules, key);
  if (ptr)
    {
      min_value = lw6map_rules_get_min (sys_context, key);
      max_value = lw6map_rules_get_max (sys_context, key);
      if (value < min_value)
	{
	  lw6sys_log (sys_context, LW6SYS_LOG_WARNING,
		      _x_ ("can't set static option \"%s\" to value %d, it's too small, using min value %d"), key, value, min_value);
	  (*ptr) = min_value;
	}
      else if (value > max_value)
	{
	  lw6sys_log (sys_context, LW6SYS_LOG_WARNING,
		      _x_ ("can't set static option \"%s\" to value %d, it's too big, using max value %d"), key, value, max_value);
	  (*ptr) = max_value;
	}
      else
	{
	  (*ptr) = value;
	}
      ret = 1;
    }

  return ret;
}

/**
 * lw6map_rules_get_bool
 *
 * @sys_context: global system context
 * @rules: struct to use
 * @key: key to query
 *
 * Get the value for a given string key, as a boolean. Of course
 * you could access the member, but this function internally does
 * the conversion between readable string and actual
 * struct offset.
 *
 * Return value: boolean.
 */
int
lw6map_rules_get_bool (lw6sys_context_t * sys_context, const lw6map_rules_t * rules, const char *key)
{
  return lw6map_rules_get_int (sys_context, rules, key) ? 1 : 0;
}

/**
 * lw6map_rules_set_bool
 *
 * @rules: struct to use
 * @key: key to set
 * @value: new boolean value for key
 *
 * Set the value for a given string key, as a boolean. Of course
 * you could access the member, but this function internally does
 * the conversion between readable string and actual
 * struct offset.
 *
 * Return value: 1 on success, 0 on failure (eg key not found)
 */
int
lw6map_rules_set_bool (lw6sys_context_t * sys_context, lw6map_rules_t * rules, const char *key, int value)
{
  return lw6map_rules_set_int (sys_context, rules, key, value ? 1 : 0);
}

/**
 * lw6map_rules_clear
 *
 * @sys_context: global system context
 * @rules: struct to init
 *
 * Set rules to 0, this is not defaults, this is 0 (probably
 * unusable as a real-world setting).
 *
 * Return value: none.
 */
void
lw6map_rules_clear (lw6sys_context_t * sys_context, lw6map_rules_t * rules)
{
  if (rules)
    {
      memset (rules, 0, sizeof (lw6map_rules_t));
    }
}

/**
 * lw6map_rules_is_same
 *
 * @sys_context: global system context
 * @rules_a: first item to compare
 * @rules_b: second item to compare
 *
 * Compares two rules items. Will tell if they contain the same data.
 *
 * Return value: 1 if same, 0 if different.
 */
int
lw6map_rules_is_same (lw6sys_context_t * sys_context, const lw6map_rules_t * rules_a, const lw6map_rules_t * rules_b)
{
  int ret = 1;

  ret = !memcmp ((char *) rules_a, (char *) rules_b, sizeof (lw6map_rules_t));

  return ret;
}

/**
 * lw6map_rules_sanity_check
 *
 * @sys_context: global system context
 * @rules: rules to check.
 *
 * Check wether the rules are within the acceptable range.
 *
 * Return value: 1 if same, 0 if different.
 */
int
lw6map_rules_sanity_check (lw6sys_context_t * sys_context, const lw6map_rules_t * rules)
{
  int i = 0;
  int value = 0;
  int min_value = 0;
  int max_value = 0;
  const char *key = NULL;
  int ret = 1;
  const int32_t *ptr = NULL;

  while ((key = LW6MAP_RULES_LIST[i]) != NULL)
    {
      ptr = _get_rules_int_ptr (sys_context, rules, key);
      if (ptr)
	{
	  /*
	   * We don't use lw6map_rules_get_int here, for it would
	   * automatically fix out of bound values, and the idea, here
	   * is to trap errors.
	   */
	  value = *ptr;
	  min_value = lw6map_rules_get_min (sys_context, key);
	  max_value = lw6map_rules_get_max (sys_context, key);
	  if (value < min_value || value > max_value)
	    {
	      lw6sys_log (sys_context, LW6SYS_LOG_WARNING,
			  _x_ ("value for rules key \"%s\" is out of range (value=%d min=%d max=%d)"), key, value, min_value, max_value);
	      ret = 0;
	    }
	}
      else
	{
	  lw6sys_log (sys_context, LW6SYS_LOG_WARNING, _x_ ("get get pointer for rules key \"%s\""), key);
	  ret = 0;
	}
      ++i;
    }

  return ret;
}
